@@ -1345,20 +1345,20 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
 
     @fields.depends('product', methods=['_get_tax_rule_pattern'])
     def compute_taxes(self, party):
-        taxes = []
+        taxes = set()
         pattern = self._get_tax_rule_pattern()
         for tax in self.product.customer_taxes_used:
             if party and party.customer_tax_rule:
                 tax_ids = party.customer_tax_rule.apply(tax, pattern)
                 if tax_ids:
-                    taxes.extend(tax_ids)
+                    taxes.update(tax_ids)
                 continue
-            taxes.append(tax.id)
+            taxes.add(tax.id)
         if party and party.customer_tax_rule:
             tax_ids = party.customer_tax_rule.apply(None, pattern)
             if tax_ids:
-                taxes.extend(tax_ids)
-        return taxes
+                taxes.update(tax_ids)
+        return list(taxes)
 
     @fields.depends('product', 'quantity', methods=['_get_context_sale_price'])
     def compute_unit_price(self):
@@ -1705,18 +1705,23 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
             return
         moved_quantity = 0
         for move in self.moves:
-            if move.state != 'cancelled':
+            if move.state != 'cancelled' and self.unit:
                 moved_quantity += Uom.compute_qty(
-                    move.uom, move.quantity, self.unit)
+                    move.uom, move.quantity, self.unit, round=False)
         if self.quantity < 0:
             moved_quantity *= -1
         invoiced_quantity = 0
         for invoice_line in self.invoice_lines:
-            if (not invoice_line.invoice
-                    or invoice_line.invoice.state != 'cancelled'):
+            if (
+                    (not invoice_line.invoice
+                        or invoice_line.invoice.state != 'cancelled')
+                    and self.unit and invoice_line.unit):
                 invoiced_quantity += Uom.compute_qty(
-                    invoice_line.unit, invoice_line.quantity, self.unit)
+                    invoice_line.unit, invoice_line.quantity, self.unit,
+                    round=False)
         actual_quantity = max(moved_quantity, invoiced_quantity, key=abs)
+        if self.unit:
+            actual_quantity = self.unit.round(actual_quantity)
         if self.actual_quantity != actual_quantity:
             self.actual_quantity = actual_quantity
 
